/*
 * snmpwalk wrapper
 *
 * Finds open ports by walking through the TCP and UDP MIB
 */

#include <includes.h>
#include "snmp_portscan.h"

/*
 *  Plugin functions
 */

#define EN_NAME "SNMP port scan"
#define FR_NAME "Scan de ports SNMP"

#define EN_DESC "\
This plugin runs snmpwalk(1) on the TCP and UDP MIB.\n\
See the section 'plugins options' to configure it"

#define FR_DESC "\
Ce plugin appelle snmpwalk(1) sur les MIB TCP et UDP.\n\
Allez voir la section 'plugins options' pour le configurer"


#define COPYRIGHT "This script is copyright (C) Michel Arboi - <arboi@algoriel.fr>"

#define EN_SUMMARY "Find open ports through snmpwalk"
#define FR_SUMMARY "Trouve les ports ouverts  l'aide de snmpwalk"

#define EN_FAMILY "Port scanners"
#define FR_FAMILY "Scanners de ports"


#define SNMP_COMMUNITY "Community name :"
#define SNMP_COMMUNITY_DFL "public"

#define SNMP_VERSION "SNMP protocol :"
#define SNMP_VERSION_DFL "1;2c"

#define SNMP_LAYER "SNMP transport layer :"
#define SNMP_LAYER_DFL "udp;tcp"

PlugExport int plugin_init(desc)
     struct arglist	*desc;
{
  /*plug_set_id(desc, 66666);*/
  plug_set_version(desc, "$Revision$");
   
  plug_set_name(desc, FR_NAME, "francais");
  plug_set_name(desc, EN_NAME, NULL);
 
  plug_set_summary(desc, FR_SUMMARY, "francais");
  plug_set_summary(desc, EN_SUMMARY, NULL);
 
  plug_set_description(desc, FR_DESC, "francais");
  plug_set_description(desc, EN_DESC, NULL);
 
  plug_set_copyright(desc, COPYRIGHT,NULL);
  plug_set_category(desc, ACT_SCANNER);
  plug_set_family(desc, FR_FAMILY, "francais");
  plug_set_family(desc, EN_FAMILY, NULL);
  plug_set_dep(desc, "ping_host.nasl");
 
  /* plug_set_timeout(desc, -1); */
  add_plugin_preference(desc, SNMP_COMMUNITY, PREF_ENTRY, SNMP_COMMUNITY_DFL);
  add_plugin_preference(desc, SNMP_VERSION, PREF_RADIO, SNMP_VERSION_DFL);
  add_plugin_preference(desc, SNMP_LAYER, PREF_RADIO, SNMP_LAYER_DFL);
  return(0);
}


#define BFLAGS	(sizeof(int) * 8)

static int
read_snmpwalk_output(desc, fp, flags)
     struct arglist	*desc;
     FILE		*fp;
     int		*flags;
{
  int	count = 0, n;
  char	s[256], *p;

  while ((fgets(s, sizeof(s), fp)) != NULL)
    {
      if ((p = strrchr(s, '=')) != NULL)
	{
	  n = atoi(p+1);
	  if (n > 0)
	    flags[n / BFLAGS] |= (1 << (n % BFLAGS));
	}
    }
  return count;
}

#define TCP_TRUC	"tcp.tcpConnTable.tcpConnEntry.tcpConnLocalPort"
#define UDP_TRUC	"udp.udpTable.udpEntry.udpLocalPort"
PlugExport int plugin_run(desc)
     struct arglist	*desc;
{
  struct arglist * globals = arg_get_value(desc, "globals");
  struct arglist * hostinfos = arg_get_value(desc, "HOSTNAME");

  FILE	*fp = NULL;
  char	*community = get_plugin_preference(desc, SNMP_COMMUNITY);
  char	*version = get_plugin_preference(desc, SNMP_VERSION);
  char	*layer = get_plugin_preference(desc, SNMP_LAYER);
  char	*opt, **argv, s[256], ip_str[16];
  int	open_ports[65536 / BFLAGS], i, num_tcp_ports, num_udp_ports;
  struct in_addr	*ip;
 

  if (community == NULL || community[0] == '\0')
    community = "public";	/* Most likely... */
  else
    {
      /* Validate user input : check that community name does not 
       * include special characters */
    }

  fprintf(stderr, "plugin_run\n");

  ip = plug_get_host_ip(desc);
  strcpy(ip_str, inet_ntoa(*ip));

  if (opt = strrchr(SNMPWALK, '/')) 
    ++ opt ;
  else
    opt = SNMPWALK;
 
  comm_send_status(globals, arg_get_value(hostinfos, "FQDN"),"portscan",
    0, 100);
  /* TCP "scan" */
  memset(open_ports, 0, sizeof(open_ports));

 argv = append_argv(0, opt);
 argv = append_argv(argv, ip_str);
 argv = append_argv(argv, community);
 sprintf(s, "%s.0.0.0.0", TCP_TRUC);
 argv = append_argv(argv, s);
 fp = ptyexecvp(SNMPWALK, (const char**)argv, NULL);
 destroy_argv (argv);

  if (fp == NULL)
    {
      fprintf(stderr, "ptyexecvp(snmpwalk) failed\n");
      return 0;
    }

  read_snmpwalk_output(desc, fp, open_ports);
  fclose(fp);

 argv = append_argv(0, opt);
 argv = append_argv(argv, ip_str);
 argv = append_argv(argv, community);
 sprintf(s, "%s.%s", TCP_TRUC, ip_str);
 argv = append_argv(argv, s);
 fp = ptyexecvp(SNMPWALK, (const char**)argv, NULL);
 destroy_argv (argv);

  if (fp == NULL)
    {
      fprintf(stderr, "ptyexecvp(snmpwalk) failed\n");
      return 0;
    }

  read_snmpwalk_output(desc, fp, open_ports);
  fclose(fp);

  for (i = 1, num_tcp_ports = 0; i < 65536; i ++)
    if (open_ports[i / BFLAGS] & (1 << (i % BFLAGS)))
      {
	num_tcp_ports ++;
	scanner_add_port(desc, i, "tcp");
#if 0
	fprintf(stderr, "TCP:%d\n", i);
#endif
      }

  /* UDP "scan" */
  memset(open_ports, 0, sizeof(open_ports));

 argv = append_argv(0, opt);
 argv = append_argv(argv, ip_str);
 argv = append_argv(argv, community);
 sprintf(s, "%s.0.0.0.0", UDP_TRUC);
 argv = append_argv(argv, s);
 fp = ptyexecvp(SNMPWALK, (const char**)argv, NULL);
 destroy_argv (argv);

  if (fp == NULL)
    {
      fprintf(stderr, "ptyexecvp(snmpwalk) failed\n");
      return 0;
    }

  read_snmpwalk_output(desc, fp, open_ports);
  fclose(fp);

 argv = append_argv(0, opt);
 argv = append_argv(argv, ip_str);
 argv = append_argv(argv, community);
 sprintf(s, "%s.%s", UDP_TRUC, ip_str);
 argv = append_argv(argv, s);
 fp = ptyexecvp(SNMPWALK, (const char**)argv, NULL);
 destroy_argv (argv);

  if (fp == NULL)
    {
      fprintf(stderr, "ptyexecvp(snmpwalk) failed\n");
      return 0;
    }

  read_snmpwalk_output(desc, fp, open_ports);
  fclose(fp);

  for (i = 1, num_udp_ports = 0; i < 65536; i ++)
    if (open_ports[i / BFLAGS] & (1 << (i % BFLAGS)))
      {
	num_udp_ports ++;
	scanner_add_port(desc, i, "udp");
#if 0
	fprintf(stderr, "UDP:%d\n", i);
#endif
      }

  /* ******** */

  comm_send_status(globals, arg_get_value(hostinfos, "FQDN"), "portscan",
		   100, 100);

  if (num_tcp_ports > 0 || num_udp_ports > 0)
    plug_set_key (desc, "Host/scanned", ARG_STRING, "1");
  if (num_udp_ports > 0)
    plug_set_key (desc, "Host/udp_scanned", ARG_STRING, "1");
  return 0;
}


